unit JTAGdataProvider;

// - VCC -
// USB data provider FPGA programmer
// For Spartan-3(E) FPGAs
// - 2010 -

interface

type
  ArrayOf61Chars = array[0 .. 60] of Byte;
  PArrayOf61Chars = ^ArrayOf61Chars;

function OpenTheTibFile(FileName: string): Boolean;
function ReadFromTibFile(var LastBlock: Boolean; var AbortProgramming: Boolean): string;
procedure ReadFromTibFileAsAStream(ReturnData: PArrayOf61Chars);
procedure CloseTheTibFile;

implementation

uses
  SysUtils;

var
  FTibFile: File;
  FTibFileWasOpen: Boolean = False;
  PositionInFile: Integer;

function OpenTheTibFile(FileName: string): Boolean;
begin
  Result := False;
  if not FileExists(FileName) then
    Exit;

  AssignFile(FTibFile, FileName);
  Reset(FTibFile, 1);
    
  Result := True;
  FTibFileWasOpen := True;

  PositionInFile := 0;
end;

function ReadFromTibFile(var LastBlock: Boolean; var AbortProgramming: Boolean): string;
var
  s: string;
  a: array[1..60] of Char;
  ActualNumberOfBytes: Integer;
begin
  AbortProgramming := False;
  try
    BlockRead(FTibFile, a, 60, ActualNumberOfBytes);

    s := a;

    PositionInFile := PositionInFile + ActualNumberOfBytes;

    SetLength(s, ActualNumberOfBytes);
    LastBlock := Eof(FTibFile);

    Result := s;
  except
    AbortProgramming := True;
    Result := #255;
  end;
end;

procedure ReadFromTibFileAsAStream(ReturnData: PArrayOf61Chars);
var
  a: array[1..60] of Byte;
  ActualNumberOfBytes: Integer;
  DataLength: Byte;
  LastBlock: Boolean;
  i: Byte;
begin
  try
    BlockRead(FTibFile, a, 60, ActualNumberOfBytes);

    PositionInFile := PositionInFile + ActualNumberOfBytes;

    LastBlock := Eof(FTibFile);
    DataLength := ActualNumberOfBytes;

    for i := 1 to DataLength do
      ReturnData^[i] := a[i];

    if LastBlock then
      DataLength := DataLength or 128;

    ReturnData^[0] := DataLength;  
  except
    ReturnData^[0] := 255; 
  end;
end;

procedure CloseTheTibFile;
begin
  if FTibFileWasOpen then
  begin
    CloseFile(FTibFile);
    FTibFileWasOpen := False;
  end;
end;

end.
